import { createRoot } from "react-dom/client";
import App from "./App";
import "./index.css";

// Register service worker for better caching
if ('serviceWorker' in navigator && import.meta.env.PROD) {
  window.addEventListener('load', () => {
    navigator.serviceWorker.register('/sw.js')
      .catch(() => {
        // Silently fail - service worker registration is optional
      });
  });
}

// Optimize for desktop performance and defer non-critical CSS
if (window.innerWidth > 1024) {
  // Preload critical fonts for desktop
  const fontPreload = document.createElement('link');
  fontPreload.rel = 'preload';
  fontPreload.as = 'font';
  fontPreload.type = 'font/woff2';
  fontPreload.crossOrigin = 'anonymous';
  fontPreload.href = '/fonts/inter-var.woff2';
  document.head.appendChild(fontPreload);
}

// CSS optimization to reduce unused rules and defer non-critical styles
const optimizeCSS = () => {
  // Defer non-critical CSS loading until after initial render
  const deferNonCriticalCSS = () => {
    const nonCriticalStyles = document.createElement('style');
    nonCriticalStyles.id = 'deferred-css';
    nonCriticalStyles.innerHTML = `
      /* Dashboard and complex UI styles - loaded after initial render */
      .transition-all { transition: all 0.15s ease-in-out; }
      .hover\\:opacity-80:hover { opacity: 0.8; }
      .hover\\:bg-gray-100:hover { background-color: rgb(243, 244, 246); }
      .focus\\:outline-none:focus { outline: none; }
      .focus\\:ring-2:focus { box-shadow: 0 0 0 2px rgba(59, 130, 246, 0.5); }
      @media (min-width: 768px) {
        .md\\:flex { display: flex; }
        .md\\:hidden { display: none; }
        .md\\:block { display: block; }
      }
      @media (min-width: 1024px) {
        .lg\\:px-8 { padding-left: 2rem; padding-right: 2rem; }
        .lg\\:py-6 { padding-top: 1.5rem; padding-bottom: 1.5rem; }
      }
    `;
    
    // Use requestIdleCallback to avoid blocking critical rendering
    if ('requestIdleCallback' in window) {
      requestIdleCallback(() => document.head.appendChild(nonCriticalStyles));
    } else {
      setTimeout(() => document.head.appendChild(nonCriticalStyles), 100);
    }
  };

  // Remove duplicate CSS rules to reduce bytes
  const removeDuplicateRules = () => {
    const seenRules = new Set();
    const sheets = Array.from(document.styleSheets);
    
    sheets.forEach(sheet => {
      try {
        if (sheet.href && sheet.href.includes(window.location.origin)) {
          const rules = Array.from(sheet.cssRules || []);
          rules.forEach((rule, index) => {
            if (seenRules.has(rule.cssText)) {
              sheet.deleteRule(index);
            } else {
              seenRules.add(rule.cssText);
            }
          });
        }
      } catch (e) {
        // Cross-origin stylesheets are handled by build process
      }
    });
  };

  // Defer non-critical CSS immediately
  deferNonCriticalCSS();
  
  // Remove duplicates after full page load
  window.addEventListener('load', () => {
    setTimeout(removeDuplicateRules, 2000);
  });
};

// Defer non-critical JavaScript modules more aggressively
const deferJavaScript = () => {
  // Only preload during extended idle periods to avoid initial bundle impact
  if ('requestIdleCallback' in window) {
    requestIdleCallback(() => {
      // Defer chart loading completely until analytics page is accessed
      const deferredModules = [
        // Heavy form validation only when forms are used
        () => import('@hookform/resolvers/zod'),
        // Complex UI components only when menus are opened
        () => import('@radix-ui/react-dropdown-menu'),
        () => import('@radix-ui/react-dialog'),
        () => import('@radix-ui/react-select'),
        () => import('@radix-ui/react-popover'),
        // Date picker only when date fields are used
        () => import('react-day-picker'),
        // Stripe only when payment is initiated
        () => import('@stripe/stripe-js')
      ];
      
      // Load these one by one with delays to avoid blocking
      deferredModules.forEach((loader, index) => {
        setTimeout(() => {
          loader().catch(() => {
            // Silent fail - modules load when needed
          });
        }, index * 500);
      });
    }, { timeout: 10000 }); // Wait 10 seconds before any preloading
  }
};

// Run optimizations
optimizeCSS();
deferJavaScript();

createRoot(document.getElementById("root")!).render(<App />);
